# Lantana CodeBlock CopyBtn拡張機能
# この拡張機能はLantanaのコードブロック中にコピーボタンを追加します
from markdown import Extension
from markdown.postprocessors import Postprocessor
from bs4 import BeautifulSoup

class CodeBlockCopyBtnExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.postprocessors.add('codeblock_copybtn', CodeBlockCopyBtnProcessor(self), '>raw_html')


class CodeBlockCopyBtnProcessor(Postprocessor):

    def run(self, html):
        return editRawHtml(html)

def editRawHtml(html):
    soup = BeautifulSoup(html, 'html5lib')
    
    # コードブロックつかうコピーボタンのアイコン定義
    ICON_COPY = 'bi-clipboard'
    ICON_COPY_CHECK = 'bi-clipboard-check'

    # コードブロックがあった場合にtrue
    hasCodeBlock = False
    
    for hlight in soup.select('.highlight'):
        hasCodeBlock = True
        hlight.insert(0,BeautifulSoup('<button class="btn copybtn position-absolute top-0 end-0"><i class="bi '+ICON_COPY+'"></i></button>','html.parser'))

    # コードブロックがあった場合はスクリプトとかを生成
    if hasCodeBlock:
        addClass(soup,'.highlight',['position-relative'])
        rawcode = """$('.copybtn').click(function (){
    const selection = window.getSelection();
    const code = this.parentNode;
    selection.selectAllChildren(code.children[code.children.length-1]);
    document.execCommand('copy');
    this.querySelector('.ICON_COPY').classList.add('text-success');
    this.querySelector('.ICON_COPY').classList.replace('ICON_COPY','ICON_COPY_CHECK');
});"""
        rawcode = rawcode.replace("ICON_COPY_CHECK",ICON_COPY_CHECK).replace("ICON_COPY",ICON_COPY)
        soup.append(BeautifulSoup(f"<script>{rawcode}</script>"))

    return str(soup)

def addClass(soup,selector,ar):
    for tag in soup.select(selector):
        tag.attrs.setdefault('class', list())
        tag["class"] += ar

def makeExtension(*args, **kwargs):
    return CodeBlockCopyBtnExtension(*args, **kwargs)


# Lantana CSS調整拡張機能
# この拡張機能は従来theme.jsで行っていたCSS関連の調整を事前に行います
from markdown import Extension
from markdown.preprocessors import Preprocessor
from markdown.postprocessors import Postprocessor
from bs4 import BeautifulSoup
import re

class LantanaExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.preprocessors.add('lantana-Pre',LantanaPreProcessor(self),'>html_block')
        md.postprocessors.add('lantana-Post', LantanaPostprocesser(self), '>raw_html')

class LantanaPreProcessor(Preprocessor):

    _pattern = re.compile(r"> \[\!(.*)] *(.*)")

    def run(self, lines: list):
        for i in range(len(lines)):
            matches = re.findall(self._pattern,lines[i])
            if len(matches) > 0:
                lines[i] = f'> []({matches[0][1].lower()}){{: .lantana-fence-tmp .{matches[0][0].lower()} }}'
        return lines

class LantanaPostprocesser(Postprocessor):

    def run(self, html):
        return editRawHtml(html)

def editRawHtml(html):
    soup = BeautifulSoup(html, 'html.parser')

    # Bootstrapクラス追加処理
    addClass(soup,'table:not(.disable-lantana)',['table'])
    addClass(soup,'img:not(.disable-lantana)',['img-fluid'])
    addClass(soup,'blockquote:not(.disable-lantana)',['blockquote'])

    return str(soup)

def addClass(soup,selector,ar):
    for tag in soup.select(selector):
        tag.attrs.setdefault('class', list())
        tag["class"] += ar

def removeClass(soup,selector,ar):
    for tag in soup.select(selector):
        tag["class"].remove(ar)

def replaceClass(soup,selector,remove,add):
    for tag in soup.select(selector):
        tag["class"].remove(remove)
        tag["class"] += add

def makeExtension(*args, **kwargs):
    return LantanaExtension(*args, **kwargs)


#!/usr/bin/env python

from __future__ import annotations

import logging
import os
import shutil
import sys
import time
import textwrap
import traceback
import warnings
import logging
from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler

import click

from mkdocs import __version__, config, utils

config_text = """site_name: MyDocs

docs_dir : 'docs'

extra_javascript:
  - https://polyfill.io/v3/polyfill.min.js?features=es6
  - https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-chtml.js

language : ja


theme: lantana

visible_search : true

plugins:
    - search:
        lang : 'ja'
        min_search_length: 2
    - awesome-pages

markdown_extensions:
    - mdx_embedly
    - attr_list
    - pymdownx.highlight:
       anchor_linenums: true
    - admonition
    - pymdownx.arithmatex:
       generic : true
    - md_in_html
    - pymdownx.details
    - pymdownx.superfences:
        custom_fences:
          - name: mermaid
            class: mermaid
            format: !!python/name:pymdownx.superfences.fence_code_format
    - pymdownx.snippets
    - pymdownx.critic
    - pymdownx.caret
    - pymdownx.keys
    - pymdownx.mark
    - pymdownx.tilde
    - pymdownx.emoji:
        emoji_index: !!python/name:materialx.emoji.twemoji
        emoji_generator: !!python/name:materialx.emoji.to_svg
    - mdx_mermaid_precompile
    - pymdownx.tasklist:
        custom_checkbox: true
    - pymdownx.magiclink
    - pymdownx.striphtml
    - mdx_lantana
"""
index_text = """---
title: Hello,World!
summary: Lantana のプロジェクトテンプレートです。ここから、あなただけのサイトを作成しましょう。
---
編集の仕方について詳しく知るには、[Lantanaのサイト](https://lantana.wsoft.ws/)を参照してください。
"""

def cli():
    """
    Lantana - Project documentation with Markdown,flavor MkDocs.
    """


def new(output_dir: str) -> None:
    docs_dir = os.path.join(output_dir, 'docs')
    config_path = os.path.join(output_dir, 'mkdocs.yml')
    index_path = os.path.join(docs_dir, 'index.md')

    if os.path.exists(config_path):
        return

    if not os.path.exists(output_dir):
        os.mkdir(output_dir)
    with open(config_path, 'w', encoding='utf-8') as f:
        f.write(config_text)

    if os.path.exists(index_path):
        return
    if not os.path.exists(docs_dir):
        os.mkdir(docs_dir)
    with open(index_path, 'w', encoding='utf-8') as f:
        f.write(index_text)



"""Create a new Lantana project"""

new('.')




if __name__ == '__main__':  # pragma: no cover
    cli()


# Lantana Accordion拡張機能
# この拡張機能はAdomonition拡張機能で作成されたアラートをLantanaでも使用できるように調整します
from markdown import Extension
from markdown.postprocessors import Postprocessor
from bs4 import BeautifulSoup
import uuid

class AccordionExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.postprocessors.add('Accordion', AccordionPostProcessor(self), '>raw_html')


class AccordionPostProcessor(Postprocessor):

    def run(self, html):
        return editRawHtml(html)

def editRawHtml(html):
    soup = BeautifulSoup(html, 'html.parser')

    for tag in soup.select('details:not(.noaccordion)'):
        createAccordion(soup,tag)
    return str(soup)

def createAccordion(soup,tag):
    
    title = tag.summary.text
    isOpen = "open" in tag.attrs and tag.attrs["open"] == "open"
    unique_id = "accordion-"+str(uuid.uuid4())
    item_id = f"{unique_id}-details"
    accordion_box = soup.new_tag("div",id = unique_id)
    accordion_item = soup.new_tag("div")
    accordion_header = soup.new_tag("h2")
    accordion_btn = soup.new_tag("button")
    tag.attrs.setdefault('class', list())
    addClass(accordion_box,tag.attrs["class"])
    addClass(accordion_box,["accordion"])
    addClass(accordion_item,["accordion-item"])
    addClass(accordion_header,["accordion-header"])
    addClass(accordion_btn,["accordion-button"])
    if not isOpen:
        addClass(accordion_btn,["collapsed"])
    accordion_btn.attrs["type"] = "button"
    accordion_btn.attrs["data-bs-toggle"] = "collapse"
    accordion_btn.attrs["aria-controls"] = item_id
    accordion_btn.attrs["data-bs-target"] = f"#{item_id}"
    accordion_btn.attrs["aria-expanded"] = str(isOpen)
    accordion_btn.append(title)
    accordion_header.append(accordion_btn)
    accordion_item.append(accordion_header)
    accordion_inner = soup.new_tag("div", id = item_id)
    addClass(accordion_inner,["accordion-collapse", "collapse"])
    if isOpen:
        addClass(accordion_inner,["show"])
    accordion_body = soup.new_tag("div")
    addClass(accordion_body,["accordion-body"])
    
    loopCount = 0
    for child in tag.children:
        if loopCount > 2:
            accordion_body.append(child)
        loopCount += 1
    accordion_inner.append(accordion_body)
    accordion_item.append(accordion_inner)
    accordion_box.append(accordion_item)
    tag.insert_before(accordion_box)
    tag.extract()
    

def addClass(tag,ar):
    tag.attrs.setdefault('class', list())
    tag["class"] += ar

def makeExtension(*args, **kwargs):
    return AccordionExtension(*args, **kwargs)


# Lantana 拡張機能
# この拡張機能はAdomonition拡張機能で作成されたアラートをLantanaでも使用できるように調整します
from markdown import Extension
from markdown.preprocessors import Preprocessor
from markdown.postprocessors import Postprocessor
from bs4 import BeautifulSoup
import re

class SelectorExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.postprocessors.add('Selector', SelectorPostProcessor(self), '>raw_html')

class SelectorPostProcessor(Postprocessor):

    def run(self, html):
        return self.editRawHtml(html)

    def editRawHtml(self,html):
        soup = BeautifulSoup(html, 'html.parser')

        for tag in soup.select('blockquote > p > .lantana-fence-tmp.selector'):
            dropdown = soup.new_tag('div',attrs={"class": ["dropdown"]})
            button = soup.new_tag('button',attrs=
                                  {"class": ["btn", "btn-secondary", "dropdown-toggle"],
                                   "type":["button"],
                                   "data-bs-toggle":["dropdown"],
                                   "aria-expanded":["false"]})
            button.string = tag["href"]
            if not button.string:
                button.string = "リンクを選択"
            ul = soup.new_tag('ul',attrs={"class": ["dropdown-menu"]})
            tag.parent.wrap(ul)
            for a in tag.parent.findChildren(name="a"):
                a.attrs.setdefault('class', list())
                if "lantana-fence-tmp" in a["class"]:
                    continue
                li = soup.new_tag('li')
                a.attrs.setdefault('class', list())
                a["class"] += ["dropdown-item"]
                a.wrap(li)
            
            ul.wrap(dropdown)
            dropdown.insert(0,button)
            dropdown.parent.unwrap()
        return str(soup)

def makeExtension(*args, **kwargs):
    return SelectorExtension(*args, **kwargs)


# Cards 拡張機能
from markdown import Extension
from markdown.postprocessors import Postprocessor
import re
import os
import glob
from natsort import natsorted

class CardsExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.postprocessors.add('cards', CardsPostprocesser(self), '>raw_html')

class Card:
    title=""
    content_dir=""
    summary=""
    date=""
    author=""
    dir_title=""
    order=""

COL_NUM = 3

def get_thumbnail_element(dir, options, index_filename='index.md',pages_filename='.pages'):
    dir_title=read_property(f'docs/{dir}/{pages_filename}','title')
    filenames = natsorted(glob.glob(f'docs/{dir}/*.md'))
    contain_index = "include-index" in options
    contain_subdir = "include-subdir" in options
    style_lite = "style-lite" in options
    cards=list()
    for i, filename in enumerate(filenames):
            card=Card()
            card.title = read_property(filename,'title')
            card.summary = read_property(filename,'summary')
            card.date = read_property(filename,'date')
            card.author = read_property(filename,'author')
            card.order = read_property(filename,'order',filename)
            card.content_dir = remove_filename(filename)
            card.dir_title=dir_title
            
            if not filename.endswith(index_filename):
                cards.append(card)
            elif contain_index:
                card.order = -1
                cards.insert(0,card)

    # 指定されている場合はサブディレクトリも見る
    if contain_subdir:
        filenames = natsorted(glob.glob(f'docs/{dir}/*/index.md'))
        for i, filename in enumerate(filenames):
            card=Card()
            card.title = read_property(filename,'title')
            card.summary = read_property(filename,'summary')
            card.date = read_property(filename,'date')
            card.author = read_property(filename,'author')
            card.order = read_property(filename,'order',filename)
            card.content_dir = remove_filename(filename)
            card.dir_title=dir_title
            cards.append(card)
    
    cards = natsorted(cards,key=lambda x:x.order)
    cards.reverse()
    
    if style_lite:
        html = '<ul>'

        for card in cards:
            html += f'<li><a href="/{card.content_dir}/">{card.title}</a></li>'

        html += '</ul>'
    else:
        html = '<div class="container-fluid">\n'
        for card in cards:
            html += '    <div class="row mb-3 card">\n'
            html += get_card_element(card.title, card.content_dir,card.summary,card.date,card.author,card.dir_title)
            html += '    </div>\n'
        html += '</div>'
    return html

def remove_filename(filename):
    """
        docs/***.mdのようなファイル名を**のみに切り出す関数
    """
    filename = filename.replace("docs/","",1)
    filename=filename[::-1].replace("dm.","",1)
    return filename[::-1]

def read_property(filename,key,default=""):
    """記事や.pagesファイルからプロパティを読みだす関数
    
    filename=ファイル名、key=プロパティの名前
    """
    with open(filename, "r", encoding="utf-8") as f:
        search=re.search(key+'\ *:\ *(.+)*\n*', f.read())
        if search != None:
            return search.group(1)
        else:
            return default

def get_card_element(title, dir,summary,date,author,dir_title):
    card = ''
    card += f'    <div class="card-header">{dir_title}</div>\n'
    card += f'      <div class="card-body">\n'
    card += f'        <h5 class="card-title">{title}</h5>\n'
    card += f'        <p class="card-text">{summary}</p>\n'
    card += f'        <a href="/{dir}/" class="btn btn-primary"><i class="bi bi-file-earmark-richtext"></i>この記事を読む</a>\n'
    card += f'      </div>\n'
    card += f'    <div class="card-footer text-muted">{author}  {date}</div>'

    return card

class CardsPostprocesser(Postprocessor):
    _pattern = re.compile("=\\\"(.*)\\\"(\\|\\[(.*)\\])?=")
    #_pattern = re.compile(r"\[WS[0-9]{5}\]")
    def run(self, html):
        html = re.sub(self._pattern, self._replace_card, html) 
        return html

    def _replace_card(self, match):
        return get_thumbnail_element(match.group(1),(match.group(3) or "").split(','))



def makeExtension(*args, **kwargs):
    return CardsExtension(*args, **kwargs)




# Lantana Alerts拡張機能
# この拡張機能はAdomonition拡張機能で作成されたアラートをLantanaでも使用できるように調整します
from markdown import Extension
from markdown.postprocessors import Postprocessor
from bs4 import BeautifulSoup

class AlertsExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.postprocessors.add('alerts', AlertsPostProcessor(self), '>raw_html')


class AlertsPostProcessor(Postprocessor):

    def run(self, html):
        return editRawHtml(html)

def editRawHtml(html):
    soup = BeautifulSoup(html, 'html.parser')

    # MaterialのAdmonitionタイトルをAlertタイトルに置換
    replaceClass(soup,'.admonition .admonition-title','admonition-title',['alert-heading'])

    # Alertにアイコンを適用
    addIcon(soup,'.admonition.note > .alert-heading','bi-check2')
    addIcon(soup,'.admonition.abstract > .alert-heading','bi-journal-text')
    addIcon(soup,'.admonition.info > .alert-heading','bi-info-circle')
    addIcon(soup,'.admonition.important > .alert-heading','bi-exclamation-circle')
    addIcon(soup,'.admonition.tip > .alert-heading','bi-lightbulb')
    addIcon(soup,'.admonition.success > .alert-heading','bi-check-all')
    addIcon(soup,'.admonition.question > .alert-heading','bi-question-circle')
    addIcon(soup,'.admonition.warning > .alert-heading','bi-exclamation-triangle')
    addIcon(soup,'.admonition.failure > .alert-heading','bi-window-x')
    addIcon(soup,'.admonition.danger > .alert-heading','bi-radioactive')
    addIcon(soup,'.admonition.caution > .alert-heading','bi-exclamation-octagon')
    addIcon(soup,'.admonition.bug > .alert-heading','bi-bug')
    addIcon(soup,'.admonition.example > .alert-heading','bi-window-sidebar')
    addIcon(soup,'.admonition.quote > .alert-heading','bi-chat-right-quote')

    # MaterialのAdmonition色をAlert色に置換
    replaceClass(soup,'.admonition.note','admonition',['alert','alert-primary'])
    replaceClass(soup,'.admonition.abstract','admonition',['alert','alert-secondary'])
    replaceClass(soup,'.admonition.info','admonition',['alert','alert-info'])
    replaceClass(soup,'.admonition.important','admonition',['alert','alert-info'])
    replaceClass(soup,'.admonition.tip','admonition',['alert','alert-warning'])
    replaceClass(soup,'.admonition.success','admonition',['alert','alert-success'])
    replaceClass(soup,'.admonition.question','admonition',['alert','alert-secondary'])
    replaceClass(soup,'.admonition.warning','admonition',['alert','alert-warning'])
    replaceClass(soup,'.admonition.failure','admonition',['alert','alert-danger'])
    replaceClass(soup,'.admonition.caution','admonition',['alert','alert-danger'])
    replaceClass(soup,'.admonition.danger','admonition',['alert','alert-danger'])
    replaceClass(soup,'.admonition.bug','admonition',['alert','alert-warning'])
    replaceClass(soup,'.admonition.example','admonition',['alert','alert-dark'])
    replaceClass(soup,'.admonition.quote','admonition',['alert','alert-light'])

    # Bootstrapのalert-linkを適用
    addClass(soup,'.alert a',['alert-link'])
    return str(soup)

def addIcon(soup,selector,iconPoint):
    for tag in soup.select(selector):
            alert_heading_icon_tag = soup.new_tag('i')
            alert_heading_icon_tag.attrs["class"] = ["bi",iconPoint]
            if tag.string:
                tag.string = ' ' + tag.string
            tag.insert(0,alert_heading_icon_tag)


def addClass(soup,selector,ar):
    for tag in soup.select(selector):
        tag.attrs.setdefault('class', list())
        tag["class"] += ar

def replaceClass(soup,selector,remove,add):
    for tag in soup.select(selector):
        tag["class"].remove(remove)
        tag["class"] += add

def makeExtension(*args, **kwargs):
    return AlertsExtension(*args, **kwargs)


# Lantana CSS調整拡張機能
# この拡張機能は従来theme.jsで行っていたCSS関連の調整を事前に行います
from markdown import Extension
from markdown.postprocessors import Postprocessor
from bs4 import BeautifulSoup
import json
import base64
import zlib
import requests

class MermaidPrecompileExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.postprocessors.add('mermaid_precompile', MermaidPrecompilePostprocesser(self), '>raw_html')


class MermaidPrecompilePostprocesser(Postprocessor):

    def run(self, html):
        return editRawHtml(html)

def editRawHtml(html):
    soup = BeautifulSoup(html, 'html.parser')

    for tag in soup.select('.mermaid'):
        mermaid_code = tag.string
        tag.clear()
        tag.insert(0,BeautifulSoup(getSvg(mermaid_code)))

    return str(soup)

def getSvg(code):
    state = {
    'code': code,
    }
    ascii_bytes = json.dumps(state).encode("ascii")
    compressed = zlib.compress(ascii_bytes)
    base64_string = base64.urlsafe_b64encode(compressed).decode('ascii')
    request_url = f'https://mermaid.ink/svg/pako:{base64_string}'
    print(f'[MermaidPreCompiler] Requesting `{request_url}`')
    return requests.get(request_url).text

def makeExtension(*args, **kwargs):
    return MermaidPrecompileExtension(*args, **kwargs)


# Lantana Link OpenNewTab拡張機能
# この拡張機能は外部サイトへのリンクを新しいタブで開きます
from markdown import Extension
from markdown.postprocessors import Postprocessor
from bs4 import BeautifulSoup

class OpenNewTabExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.postprocessors.add('link_opennewtab', OpenNewTabPostprocesser(self), '>raw_html')


class OpenNewTabPostprocesser(Postprocessor):

    def run(self, html):
        return editRawHtml(html)

def editRawHtml(html):
    soup = BeautifulSoup(html, 'html.parser')

    for tag in soup.select('a[href*="://"]:not(.unnewtab)'):
        tag.attrs["target"] = ['_blank']
        tag.attrs["rel"] = ['noopener" ,"noreferrer']

    return str(soup)

def makeExtension(*args, **kwargs):
    return OpenNewTabExtension(*args, **kwargs)


from setuptools import setup, find_packages

VERSION = '2.9.8.7'

setup(
    name="lantana",
    version=VERSION,
    url='https://lantana.wsoft.ws/',
    license='MIT',
    description='Bootstrap theme for MkDocs',
    author='WSOFT',
    author_email='info@wsoft.ws',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[ 'markdown<3.4','mkdocs>=1.1','pymdown-extensions','mkdocs-material','mkdocs-awesome-pages-plugin>=2.3','mkdocs-macros-plugin>=0.6.3','mkdocs-git-authors-plugin>=0.6.2','mkdocs-mermaid2-plugin>=0.5.0','mkdocs-git-revision-date-plugin>=0.3.1','natsort>=8.3.1',"beautifulsoup4>=4.12.3","html5lib>=1.1"],
    python_requires='>=3.5',
    entry_points={
        'mkdocs.themes': [
            'lantana = lantana',
        ],
        "console_scripts":[
            'lantana = lantana.__main__:cli',
        ],
        "markdown.extensions": [
            "mdx_wsid = lantana.extensions.mdx_wsid:WSIDExtension",
            "mdx_embedly = lantana.extensions.mdx_embedly:EmbedlyExtension",
            "lantana = lantana.extensions.lantana:LantanaExtension",
            "lantana.cards = lantana.extensions.cards:CardsExtension",
            "lantana.alerts = lantana.extensions.alerts:AlertsExtension",
            "lantana.alerts2 = lantana.extensions.alerts2:Alerts2Extension",
            "lantana.selector = lantana.extensions.selector:SelectorExtension",
            "lantana.accordion = lantana.extensions.accordion:AccordionExtension",
            "lantana.link_opennewtab = lantana.extensions.link_opennewtab:OpenNewTabExtension",
            "lantana.codeblock_copybtn = lantana.extensions.codeblock_copybtn:CodeBlockCopyBtnExtension",
            "lantana.mermaid_precompile = lantana.extensions.mermaid_precompile:MermaidPrecompileExtension",
        ]
    },
    zip_safe=False
)


# Lantana 拡張機能
# この拡張機能はAdomonition拡張機能で作成されたアラートをLantanaでも使用できるように調整します
from markdown import Extension
from markdown.preprocessors import Preprocessor
from markdown.postprocessors import Postprocessor
from bs4 import BeautifulSoup
import re

class Alerts2Extension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.postprocessors.add('Alerts2-Post', Alerts2PostProcessor(self), '>raw_html')

    
class Alerts2PostProcessor(Postprocessor):

    def run(self, html):
        return self.editRawHtml(html)

    def editRawHtml(self,html):
        soup = BeautifulSoup(html, 'html.parser')

        for tag in soup.select('blockquote > p > .lantana-fence-tmp'):
            alert_tag = soup.new_tag('div')
            alert_tag.attrs.setdefault('class', list())
            alert_tag["class"] += ['alert']
            tagname = ""
            iconPoint = ""
            processed = False
            for name in tag["class"]:
                tagname = name
                if name == "note":
                    iconPoint = "bi-check2"
                    alert_tag["class"] += ['alert-primary']
                    processed = True
                if name == "question":
                    iconPoint = "bi-question-circle"
                    alert_tag["class"] += ['alert-secondary']
                    processed = True
                if name == "abstract":
                    iconPoint = "bi-journal-text"
                    alert_tag["class"] += ['alert-secondary']
                    processed = True
                if name == "info":
                    iconPoint = "bi-info-circle"
                    alert_tag["class"] += ['alert-info']
                    processed = True
                if name == "important":
                    iconPoint = "bi-exclamation-circle"
                    alert_tag["class"] += ['alert-info']
                    processed = True
                if name == "tip":
                    iconPoint = "bi-lightbulb"
                    alert_tag["class"] += ['alert-warning']
                    processed = True
                if name == "warning":
                    iconPoint = "bi-exclamation-triangle"
                    alert_tag["class"] += ['alert-warning']
                    processed = True
                if name == "bug":
                    iconPoint = "bi-bug"
                    alert_tag["class"] += ['alert-warning']
                    processed = True
                if name == "failure":
                    iconPoint = "bi-window-x"
                    alert_tag["class"] += ['alert-danger']
                    processed = True
                if name == "danger":
                    iconPoint = "bi-radioactive"
                    alert_tag["class"] += ['alert-danger']
                    processed = True
                if name == "caution":
                    iconPoint = "bi-exclamation-octagon"
                    alert_tag["class"] += ['alert-danger']
                    processed = True
                if name == "success":
                    iconPoint = "bi-check-all"
                    alert_tag["class"] += ['alert-success']
                    processed = True
                if name == "example":
                    iconPoint = "bi-window-sidebar"
                    alert_tag["class"] += ['alert-dark']
                    processed = True
                if name == "quote":
                    iconPoint = "bi-chat-right-quote"
                    alert_tag["class"] += ['alert-light']
                    processed = True
            if not processed:
                continue
            tag.parent.parent.wrap(alert_tag)
            tag.parent.parent.unwrap()
            if not tag["href"]:
                tag["href"] = tagname.capitalize()
            alert_heading_tag = soup.new_tag('p')
            alert_heading_icon_tag = soup.new_tag('i')
            alert_heading_icon_tag.attrs["class"] = ["bi",iconPoint]
            alert_heading_tag.string = ' ' + tag["href"]
            alert_heading_tag.insert(0,alert_heading_icon_tag)
            alert_heading_tag.attrs["class"] = ["alert-heading"]
            alert_tag.insert(0,alert_heading_tag)
            tag.extract()
        addClass(soup,'.alert a',['alert-link'])
        return str(soup)

def addClass(soup,selector,ar):
        for tag in soup.select(selector):
            tag.attrs.setdefault('class', list())
            tag["class"] += ar
def makeExtension(*args, **kwargs):
    return Alerts2Extension(*args, **kwargs)